use {
    crate::prelude::ParseRequestError,
    std::{fmt, io, string::FromUtf8Error, time::SystemTimeError},
};

#[derive(Debug)]
pub enum Error {
    Io(io::Error),
    Request(ParseRequestError),
    Storage(String),
    Rustls(rustls::Error),
    Time(SystemTimeError),
    Utf8,
}

impl fmt::Display for Error {
    fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
        match self {
            Self::Io(e) => write!(f, "{e}"),
            Self::Storage(e) => write!(f, "{e}"),
            Self::Request(e) => write!(f, "{e}"),
            Self::Rustls(e) => write!(f, "{e}"),
            Self::Time(e) => write!(f, "{e}"),
            Self::Utf8 => write!(f, "Utf8 error"),
        }
    }
}

impl std::error::Error for Error {
    fn source(&self) -> Option<&(dyn std::error::Error + 'static)> {
        match self {
            Self::Io(e) => Some(e),
            Self::Request(e) => Some(e),
            Self::Rustls(e) => Some(e),
            Self::Time(e) => Some(e),
            _ => None,
        }
    }
}

impl From<io::Error> for Error {
    fn from(value: io::Error) -> Self {
        Self::Io(value)
    }
}

impl From<ParseRequestError> for Error {
    fn from(value: ParseRequestError) -> Self {
        Self::Request(value)
    }
}

impl From<rustls::Error> for Error {
    fn from(value: rustls::Error) -> Self {
        Self::Rustls(value)
    }
}

impl From<SystemTimeError> for Error {
    fn from(value: SystemTimeError) -> Self {
        Self::Time(value)
    }
}

impl From<FromUtf8Error> for Error {
    fn from(_value: FromUtf8Error) -> Self {
        Self::Utf8
    }
}
